# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2010
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill
# eole@ac-dijon.fr
###########################################################################
"""
Gestion et application des modèles de partage Samba
"""
from glob import glob
from os import listdir
from os.path import join, isfile
from creole import parsedico
from creole.config import containers_file
from pyeole.service import service_code

## chemins des containers
dico_container = {}
if isfile(containers_file):
    execfile(containers_file, {}, dico_container)
if dico_container.get('container_path_fichier', '') != '':
    CONTAINER_PATH_FICHIER = dico_container['container_path_fichier']
else:
    CONTAINER_PATH_FICHIER = '/'
## variables du dictionnaire
dico = parsedico.parse_dico()

def get_share_models():
    """
    retourne la liste des modèles de partage disponibles
    """
    templates = []
    files = listdir('/usr/share/eole/fichier/models/')
    for fil in files:
        if fil.endswith('.tmpl'):
            templates.append(fil[0:-5])
    if 'standard' not in templates:
        # il en faut au moins un
        templates.append('standard')
    templates.sort()
    return templates


def synchronize(shares, restart=True):
    """
    Ajout des partages ldap dans le fichier smb.conf
    @shares : liste de dictionnaires contenants les clés :
        name, path, group, desc, model
    """
    fic = open('/var/lib/eole/global_smb.tmpl')
    smb = [fic.read()]
    fic.close()

    # inclusion des fichiers /etc/samba/conf.d/*.conf
    smb.append('# Inclusions depuis "/etc/samba/conf.d/"\n')
    for include_file in glob(join(CONTAINER_PATH_FICHIER, 'etc/samba/conf.d/*.conf')):
        if CONTAINER_PATH_FICHIER != '/':
            include_file = include_file.replace(CONTAINER_PATH_FICHIER, '')
        smb.append('include = %s\n' % include_file)
    smb.append('\n')

    templates = {}
    for tmpl in get_share_models():
        try:
            fic = open('/usr/share/eole/fichier/models/%s.tmpl' % tmpl)
            templates[tmpl] = fic.read()
            fic.close()
        except:
            print "Erreur de lecture du template %s" % tmpl

    default = """[%(name)s]
        inherit permissions = no
        inherit acls = yes
        read only = no
        preserve case = no
        create mask = 644
        directory mask = 755
        Write list = @%(group)s
        Valid Users = @%(group)s @DomainAdmins
        path = %(path)s
        comment = %(desc)s
        Volume = %(desc)s
"""
    #result = ldapsearch(share_filters)
    #for share in result:
    #    dico_share = {}
    #    dico_share['name'] = share[1]['sambaShareName'][0]
    #    dico_share['path'] = share[1]['sambaFilePath'][0]
    #    dico_share['group'] = share[1]['sambaShareGroup'][0]
    #    dico_share['desc'] = share[1]['description'][0]
    for dico_share in shares:
        # Merge dico to use eole variables
        for key in dico:
            if not dico_share.has_key(key):
                dico_share[key] = dico[key]
        model = dico_share.get('model', '')
        if model == '':
            model = 'standard'
            print "modèle non défini pour le partage %s" % dico_share['name']

        ## FIXME : création du répertoire si manquant
        #if not dico_share['path'].startswith('%') and is_group(dico_share['group']):
        #    if not isdir(dico_share['path']) or not verif_acl(dico_share['path'], dico_share['group']):
        #        system("mkdir -p %(path)s;chmod 770 %(path)s" % dico_share)
        #        if share[1].has_key('sambaShareOptions'):
        #            if 'sticky' in share[1]['sambaShareOptions']:
        #                system ("chgrp %(group)s %(path)s;chmod 1770 %(path)s;" % dico_share)
        #        set_acl(dico_share['path'], 'root', dico_share['group'])

        ## génération du template pour samba
        try:
            smb.append(templates[model] % dico_share)
        except KeyError:
            print "Le modèle de partage %s n'existe pas" % model
            smb.append(default % dico_share)
        except:
            print "Erreur lors de la templatisation du modèle %s" % model
            smb.append(default % dico_share)
        smb.append('\n')

    # écriture du fichier final
    fic = open(join(CONTAINER_PATH_FICHIER, 'etc/samba/smb.conf'), 'w')
    fic.writelines(smb)
    fic.close()

    # FIXME : traitements spécifiques à Horus + imprimantes

    if restart:
        service_code('smbd', 'restart', 'fichier')
